@@ -140,6 +140,11 @@ val lookup_immediate : string -> pdfobject -> pdfobject option
     "/RoleMap"]] *)
 val lookup_chain : t -> pdfobject -> string list -> pdfobject option
 
+(** [replace_chain doc chain obj] sets the object at the given chain from the
+    trailer dictionary to the given object. If the final part of the chain does
+    not exist, it is created as direct, nested, dictionaries. *)
+val replace_chain : t -> string list -> pdfobject -> unit
+
 (** Return the object number of an indirect dictionary object, if it is indirect. *)
 val indirect_number : t -> string -> pdfobject -> int option
 
@@ -295,9 +300,7 @@ val change_id : t -> string -> unit
 
 (**/**)
 
-val replace_chain : t -> string list -> string * pdfobject -> unit
 
-val replace_chain_all_direct : pdfobject -> string list -> string * pdfobject -> pdfobject
 
 (* This is only for the use of Pdfread for when the /Length is incorrect. *)
 type toget_crypt =
