@@ -384,6 +384,222 @@ let decode_lzw early i =
  r -- /Rows
 *)
 
+let rec write_white_code = function
+  | -1 -> [0; 0; 0; 0; 0; 0; 0; 0; 0; 0; 0; 1]
+  | 0 -> [0; 0; 1; 1; 0; 1; 0; 1]
+  | 1 -> [0; 0; 0; 1; 1; 1]
+  | 2 -> [0; 1; 1; 1]
+  | 3 -> [1; 0; 0; 0]
+  | 4 -> [1; 0; 1; 1]
+  | 5 -> [1; 1; 0; 0]
+  | 6 -> [1; 1; 1; 0]
+  | 7 -> [1; 1; 1; 1]
+  | 8 -> [1; 0; 0; 1; 1]
+  | 9 -> [1; 0; 1; 0; 0]
+  | 10 -> [0; 0; 1; 1; 1]
+  | 11 -> [0; 1; 0; 0; 0]
+  | 12 -> [0; 0; 1; 0; 0; 0]
+  | 13 -> [0; 0; 0; 0; 1; 1]
+  | 14 -> [1; 1; 0; 1; 0; 0]
+  | 15 -> [1; 1; 0; 1; 0; 1]
+  | 16 -> [1; 0; 1; 0; 1; 0]
+  | 17 -> [1; 0; 1; 0; 1; 1]
+  | 18 -> [0; 1; 0; 0; 1; 1; 1]
+  | 19 -> [0; 0; 0; 1; 1; 0; 0]
+  | 20 -> [0; 0; 0; 1; 0; 0; 0]
+  | 21 -> [0; 0; 1; 0; 1; 1; 1]
+  | 22 -> [0; 0; 0; 0; 0; 1; 1]
+  | 23 -> [0; 0; 0; 0; 1; 0; 0]
+  | 24 -> [0; 1; 0; 1; 0; 0; 0]
+  | 25 -> [0; 1; 0; 1; 0; 1; 1]
+  | 26 -> [0; 0; 1; 0; 0; 1; 1]
+  | 27 -> [0; 1; 0; 0; 1; 0; 0]
+  | 28 -> [0; 0; 1; 1; 0; 0; 0]
+  | 29 -> [0; 0; 0; 0; 0; 0; 1; 0]
+  | 30 -> [0; 0; 0; 0; 0; 0; 1; 1]
+  | 31 -> [0; 0; 0; 1; 1; 0; 1; 0]
+  | 32 -> [0; 0; 0; 1; 1; 0; 1; 1]
+  | 33 -> [0; 0; 0; 1; 0; 0; 1; 0]
+  | 34 -> [0; 0; 0; 1; 0; 0; 1; 1]
+  | 35 -> [0; 0; 0; 1; 0; 1; 0; 0]
+  | 36 -> [0; 0; 0; 1; 0; 1; 0; 1]
+  | 37 -> [0; 0; 0; 1; 0; 1; 1; 0]
+  | 38 -> [0; 0; 0; 1; 0; 1; 1; 1]
+  | 39 -> [0; 0; 1; 0; 1; 0; 0; 0]
+  | 40 -> [0; 0; 1; 0; 1; 0; 0; 1]
+  | 41 -> [0; 0; 1; 0; 1; 0; 1; 0]
+  | 42 -> [0; 0; 1; 0; 1; 0; 1; 1]
+  | 43 -> [0; 0; 1; 0; 1; 1; 0; 0]
+  | 44 -> [0; 0; 1; 0; 1; 1; 0; 1]
+  | 45 -> [0; 0; 0; 0; 0; 1; 0; 0]
+  | 46 -> [0; 0; 0; 0; 0; 1; 0; 1]
+  | 47 -> [0; 0; 0; 0; 1; 0; 1; 0]
+  | 48 -> [0; 0; 0; 0; 1; 0; 1; 1]
+  | 49 -> [0; 1; 0; 1; 0; 0; 1; 0]
+  | 50 -> [0; 1; 0; 1; 0; 0; 1; 1]
+  | 51 -> [0; 1; 0; 1; 0; 1; 0; 0]
+  | 52 -> [0; 1; 0; 1; 0; 1; 0; 1]
+  | 53 -> [0; 0; 1; 0; 0; 1; 0; 0]
+  | 54 -> [0; 0; 1; 0; 0; 1; 0; 1]
+  | 55 -> [0; 1; 0; 1; 1; 0; 0; 0]
+  | 56 -> [0; 1; 0; 1; 1; 0; 0; 1]
+  | 57 -> [0; 1; 0; 1; 1; 0; 1; 0]
+  | 58 -> [0; 1; 0; 1; 1; 0; 1; 1]
+  | 59 -> [0; 1; 0; 0; 1; 0; 1; 0]
+  | 60 -> [0; 1; 0; 0; 1; 0; 1; 1]
+  | 61 -> [0; 0; 1; 1; 0; 0; 1; 0]
+  | 62 -> [0; 0; 1; 1; 0; 0; 1; 1]
+  | 63 -> [0; 0; 1; 1; 0; 1; 0; 0]
+  | n when n >= 2560 -> [0; 0; 0; 0; 0; 0; 0; 1; 1; 1; 1; 1] @ write_white_code (n - 2560)
+  | n when n >= 2496 -> [0; 0; 0; 0; 0; 0; 0; 1; 1; 1; 1; 0] @ write_white_code (n - 2496)
+  | n when n >= 2432 -> [0; 0; 0; 0; 0; 0; 0; 1; 1; 1; 0; 1] @ write_white_code (n - 2432)
+  | n when n >= 2368 -> [0; 0; 0; 0; 0; 0; 0; 1; 1; 1; 0; 0] @ write_white_code (n - 2368)
+  | n when n >= 2304 -> [0; 0; 0; 0; 0; 0; 0; 1; 0; 1; 1; 1] @ write_white_code (n - 2304)
+  | n when n >= 2240 -> [0; 0; 0; 0; 0; 0; 0; 1; 0; 1; 1; 0] @ write_white_code (n - 2240)
+  | n when n >= 2176 -> [0; 0; 0; 0; 0; 0; 0; 1; 0; 1; 0; 1] @ write_white_code (n - 2176)
+  | n when n >= 2112 -> [0; 0; 0; 0; 0; 0; 0; 1; 0; 1; 0; 0] @ write_white_code (n - 2112)
+  | n when n >= 2048 -> [0; 0; 0; 0; 0; 0; 0; 1; 0; 0; 1; 1] @ write_white_code (n - 2048)
+  | n when n >= 1984 -> [0; 0; 0; 0; 0; 0; 0; 1; 0; 0; 1; 0] @ write_white_code (n - 1984)
+  | n when n >= 1920 -> [0; 0; 0; 0; 0; 0; 0; 1; 1; 0; 1] @ write_white_code (n - 1920)
+  | n when n >= 1856 -> [0; 0; 0; 0; 0; 0; 0; 1; 1; 0; 0] @ write_white_code (n - 1856)
+  | n when n >= 1792 -> [0; 0; 0; 0; 0; 0; 0; 1; 0; 0; 0] @ write_white_code (n - 1792)
+  | n when n >= 1728 -> [0; 1; 0; 0; 1; 1; 0; 1; 1] @ write_white_code (n - 1728)
+  | n when n >= 1664 -> [0; 1; 1; 0; 0; 0] @ write_white_code (n - 1664)
+  | n when n >= 1600 -> [0; 1; 0; 0; 1; 1; 0; 1; 0] @ write_white_code (n - 1600)
+  | n when n >= 1536 -> [0; 1; 0; 0; 1; 1; 0; 0; 1] @ write_white_code (n - 1536)
+  | n when n >= 1472 -> [0; 1; 0; 0; 1; 1; 0; 0; 0] @ write_white_code (n - 1472)
+  | n when n >= 1408 -> [0; 1; 1; 0; 1; 1; 0; 1; 1] @ write_white_code (n - 1408)
+  | n when n >= 1344 -> [0; 1; 1; 0; 1; 1; 0; 1; 0] @ write_white_code (n - 1344)
+  | n when n >= 1280 -> [0; 1; 1; 0; 1; 1; 0; 0; 1] @ write_white_code (n - 1280)
+  | n when n >= 1216 -> [0; 1; 1; 0; 1; 1; 0; 0; 0] @ write_white_code (n - 1216)
+  | n when n >= 1152 -> [0; 1; 1; 0; 1; 0; 1; 1; 1] @ write_white_code (n - 1152)
+  | n when n >= 1088 -> [0; 1; 1; 0; 1; 0; 1; 1; 0] @ write_white_code (n - 1088)
+  | n when n >= 1024 -> [0; 1; 1; 0; 1; 0; 1; 0; 1] @ write_white_code (n - 1024)
+  | n when n >= 960 -> [0; 1; 1; 0; 1; 0; 1; 0; 0] @ write_white_code (n - 960)
+  | n when n >= 896 -> [0; 1; 1; 0; 1; 0; 0; 1; 1] @ write_white_code (n - 896)
+  | n when n >= 832 -> [0; 1; 1; 0; 1; 0; 0; 1; 0] @ write_white_code (n - 832)
+  | n when n >= 768 -> [0; 1; 1; 0; 0; 1; 1; 0; 1] @ write_white_code (n - 768)
+  | n when n >= 704 -> [0; 1; 1; 0; 0; 1; 1; 0; 0] @ write_white_code (n - 704)
+  | n when n >= 640 -> [0; 1; 1; 0; 0; 1; 1; 1] @ write_white_code (n - 640)
+  | n when n >= 576 -> [0; 1; 1; 0; 1; 0; 0; 0] @ write_white_code (n - 576)
+  | n when n >= 512 -> [0; 1; 1; 0; 0; 1; 0; 1] @ write_white_code (n - 512)
+  | n when n >= 448 -> [0; 1; 1; 0; 0; 1; 0; 0] @ write_white_code (n - 448)
+  | n when n >= 384 -> [0; 0; 1; 1; 0; 1; 1; 1] @ write_white_code (n - 384)
+  | n when n >= 320 -> [0; 0; 1; 1; 0; 1; 1; 0] @ write_white_code (n - 320)
+  | n when n >= 256 -> [0; 1; 1; 0; 1; 1; 1] @ write_white_code (n - 256)
+  | n when n >= 192 -> [0; 1; 0; 1; 1; 1] @ write_white_code (n - 192)
+  | n when n >= 128 -> [1; 0; 0; 1; 0] @ write_white_code (n - 128)
+  | n when n >= 64 -> [1; 1; 0; 1; 1] @ write_white_code (n - 64)
+  | _ -> raise (Pdf.PDFError "write_white_code: unknown")
+
+let rec write_black_code = function
+  | -1 -> [0; 0; 0; 0; 0; 0; 0; 0; 0; 0; 0; 1]
+  | 0 -> [0; 0; 0; 0; 1; 1; 0; 1; 1; 1]
+  | 1 -> [0; 1; 0]
+  | 2 -> [1; 1]
+  | 3 -> [1; 0]
+  | 4 -> [0; 1; 1]
+  | 5 -> [0; 0; 1; 1]
+  | 6 -> [0; 0; 1; 0]
+  | 7 -> [0; 0; 0; 1; 1]
+  | 8 -> [0; 0; 0; 1; 0; 1]
+  | 9 -> [0; 0; 0; 1; 0; 0]
+  | 10 -> [0; 0; 0; 0; 1; 0; 0]
+  | 11 -> [0; 0; 0; 0; 1; 0; 1]
+  | 12 -> [0; 0; 0; 0; 1; 1; 1]
+  | 13 -> [0; 0; 0; 0; 0; 1; 0; 0]
+  | 14 -> [0; 0; 0; 0; 0; 1; 1; 1]
+  | 15 -> [0; 0; 0; 0; 1; 1; 0; 0; 0]
+  | 16 -> [0; 0; 0; 0; 0; 1; 0; 1; 1; 1]
+  | 17 -> [0; 0; 0; 0; 0; 1; 1; 0; 0; 0]
+  | 18 -> [0; 0; 0; 0; 0; 0; 1; 0; 0; 0]
+  | 19 -> [0; 0; 0; 0; 1; 1; 0; 0; 1; 1; 1]
+  | 20 -> [0; 0; 0; 0; 1; 1; 0; 1; 0; 0; 0]
+  | 21 -> [0; 0; 0; 0; 1; 1; 0; 1; 1; 0; 0]
+  | 22 -> [0; 0; 0; 0; 0; 1; 1; 0; 1; 1; 1]
+  | 23 -> [0; 0; 0; 0; 0; 1; 0; 1; 0; 0; 0]
+  | 24 -> [0; 0; 0; 0; 0; 0; 1; 0; 1; 1; 1]
+  | 25 -> [0; 0; 0; 0; 0; 0; 1; 1; 0; 0; 0]
+  | 26 -> [0; 0; 0; 0; 1; 1; 0; 0; 1; 0; 1; 0]
+  | 27 -> [0; 0; 0; 0; 1; 1; 0; 0; 1; 0; 1; 1]
+  | 28 -> [0; 0; 0; 0; 1; 1; 0; 0; 1; 1; 0; 0]
+  | 29 -> [0; 0; 0; 0; 1; 1; 0; 0; 1; 1; 0; 1]
+  | 30 -> [0; 0; 0; 0; 0; 1; 1; 0; 1; 0; 0; 0]
+  | 31 -> [0; 0; 0; 0; 0; 1; 1; 0; 1; 0; 0; 1]
+  | 32 -> [0; 0; 0; 0; 0; 1; 1; 0; 1; 0; 1; 0]
+  | 33 -> [0; 0; 0; 0; 0; 1; 1; 0; 1; 0; 1; 1]
+  | 34 -> [0; 0; 0; 0; 1; 1; 0; 1; 0; 0; 1; 0]
+  | 35 -> [0; 0; 0; 0; 1; 1; 0; 1; 0; 0; 1; 1]
+  | 36 -> [0; 0; 0; 0; 1; 1; 0; 1; 0; 1; 0; 0]
+  | 37 -> [0; 0; 0; 0; 1; 1; 0; 1; 0; 1; 0; 1]
+  | 38 -> [0; 0; 0; 0; 1; 1; 0; 1; 0; 1; 1; 0]
+  | 39 -> [0; 0; 0; 0; 1; 1; 0; 1; 0; 1; 1; 1]
+  | 40 -> [0; 0; 0; 0; 0; 1; 1; 0; 1; 1; 0; 0]
+  | 41 -> [0; 0; 0; 0; 0; 1; 1; 0; 1; 1; 0; 1]
+  | 42 -> [0; 0; 0; 0; 1; 1; 0; 1; 1; 0; 1; 0]
+  | 43 -> [0; 0; 0; 0; 1; 1; 0; 1; 1; 0; 1; 1]
+  | 44 -> [0; 0; 0; 0; 0; 1; 0; 1; 0; 1; 0; 0]
+  | 45 -> [0; 0; 0; 0; 0; 1; 0; 1; 0; 1; 0; 1]
+  | 46 -> [0; 0; 0; 0; 0; 1; 0; 1; 0; 1; 1; 0]
+  | 47 -> [0; 0; 0; 0; 0; 1; 0; 1; 0; 1; 1; 1]
+  | 48 -> [0; 0; 0; 0; 0; 1; 1; 0; 0; 1; 0; 0]
+  | 49 -> [0; 0; 0; 0; 0; 1; 1; 0; 0; 1; 0; 1]
+  | 50 -> [0; 0; 0; 0; 0; 1; 0; 1; 0; 0; 1; 0]
+  | 51 -> [0; 0; 0; 0; 0; 1; 0; 1; 0; 0; 1; 1]
+  | 52 -> [0; 0; 0; 0; 0; 0; 1; 0; 0; 1; 0; 0]
+  | 53 -> [0; 0; 0; 0; 0; 0; 1; 1; 0; 1; 1; 1]
+  | 54 -> [0; 0; 0; 0; 0; 0; 1; 1; 1; 0; 0; 0]
+  | 55 -> [0; 0; 0; 0; 0; 0; 1; 0; 0; 1; 1; 1]
+  | 56 -> [0; 0; 0; 0; 0; 0; 1; 0; 1; 0; 0; 0]
+  | 57 -> [0; 0; 0; 0; 0; 1; 0; 1; 1; 0; 0; 0]
+  | 58 -> [0; 0; 0; 0; 0; 1; 0; 1; 1; 0; 0; 1]
+  | 59 -> [0; 0; 0; 0; 0; 0; 1; 0; 1; 0; 1; 1]
+  | 60 -> [0; 0; 0; 0; 0; 0; 1; 0; 1; 1; 0; 0]
+  | 61 -> [0; 0; 0; 0; 0; 1; 0; 1; 1; 0; 1; 0]
+  | 62 -> [0; 0; 0; 0; 0; 1; 1; 0; 0; 1; 1; 0]
+  | 63 -> [0; 0; 0; 0; 0; 1; 1; 0; 0; 1; 1; 1]
+  | n when n >= 2560 -> [0; 0; 0; 0; 0; 0; 0; 1; 1; 1; 1; 1] @ write_black_code (n - 2560)
+  | n when n >= 2496 -> [0; 0; 0; 0; 0; 0; 0; 1; 1; 1; 1; 0] @ write_black_code (n - 2496)
+  | n when n >= 2432 -> [0; 0; 0; 0; 0; 0; 0; 1; 1; 1; 0; 1] @ write_black_code (n - 2432)
+  | n when n >= 2368 -> [0; 0; 0; 0; 0; 0; 0; 1; 1; 1; 0; 0] @ write_black_code (n - 2368)
+  | n when n >= 2304 -> [0; 0; 0; 0; 0; 0; 0; 1; 0; 1; 1; 1] @ write_black_code (n - 2304)
+  | n when n >= 2240 -> [0; 0; 0; 0; 0; 0; 0; 1; 0; 1; 1; 0] @ write_black_code (n - 2240)
+  | n when n >= 2176 -> [0; 0; 0; 0; 0; 0; 0; 1; 0; 1; 0; 1] @ write_black_code (n - 2176)
+  | n when n >= 2112 -> [0; 0; 0; 0; 0; 0; 0; 1; 0; 1; 0; 0] @ write_black_code (n - 2112)
+  | n when n >= 2048 -> [0; 0; 0; 0; 0; 0; 0; 1; 0; 0; 1; 1] @ write_black_code (n - 2048)
+  | n when n >= 1984 -> [0; 0; 0; 0; 0; 0; 0; 1; 0; 0; 1; 0] @ write_black_code (n - 1984)
+  | n when n >= 1920 -> [0; 0; 0; 0; 0; 0; 0; 1; 1; 0; 1] @ write_black_code (n - 1920)
+  | n when n >= 1856 -> [0; 0; 0; 0; 0; 0; 0; 1; 1; 0; 0] @ write_black_code (n - 1856)
+  | n when n >= 1792 -> [0; 0; 0; 0; 0; 0; 0; 1; 0; 0; 0] @ write_black_code (n - 1792)
+  | n when n >= 1728 -> [0; 0; 0; 0; 0; 0; 1; 1; 0; 0; 1; 0; 1] @ write_black_code (n - 1728)
+  | n when n >= 1664 -> [0; 0; 0; 0; 0; 0; 1; 1; 0; 0; 1; 0; 0] @ write_black_code (n - 1664)
+  | n when n >= 1600 -> [0; 0; 0; 0; 0; 0; 1; 0; 1; 1; 0; 1; 1] @ write_black_code (n - 1600)
+  | n when n >= 1536 -> [0; 0; 0; 0; 0; 0; 1; 0; 1; 1; 0; 1; 0] @ write_black_code (n - 1536)
+  | n when n >= 1472 -> [0; 0; 0; 0; 0; 0; 1; 0; 1; 0; 1; 0; 1] @ write_black_code (n - 1472)
+  | n when n >= 1408 -> [0; 0; 0; 0; 0; 0; 1; 0; 1; 0; 1; 0; 0] @ write_black_code (n - 1408)
+  | n when n >= 1344 -> [0; 0; 0; 0; 0; 0; 1; 0; 1; 0; 0; 1; 1] @ write_black_code (n - 1344)
+  | n when n >= 1280 -> [0; 0; 0; 0; 0; 0; 1; 0; 1; 0; 0; 1; 0] @ write_black_code (n - 1280)
+  | n when n >= 1216 -> [0; 0; 0; 0; 0; 0; 1; 1; 1; 0; 1; 1; 1] @ write_black_code (n - 1216)
+  | n when n >= 1152 -> [0; 0; 0; 0; 0; 0; 1; 1; 1; 0; 1; 1; 0] @ write_black_code (n - 1152)
+  | n when n >= 1088 -> [0; 0; 0; 0; 0; 0; 1; 1; 1; 0; 1; 0; 1] @ write_black_code (n - 1088)
+  | n when n >= 1024 -> [0; 0; 0; 0; 0; 0; 1; 1; 1; 0; 1; 0; 0] @ write_black_code (n - 1024)
+  | n when n >= 960 -> [0; 0; 0; 0; 0; 0; 1; 1; 1; 0; 0; 1; 1] @ write_black_code (n - 960)
+  | n when n >= 896 -> [0; 0; 0; 0; 0; 0; 1; 1; 1; 0; 0; 1; 0] @ write_black_code (n - 896)
+  | n when n >= 832 -> [0; 0; 0; 0; 0; 0; 1; 0; 0; 1; 1; 0; 1] @ write_black_code (n - 832)
+  | n when n >= 768 -> [0; 0; 0; 0; 0; 0; 1; 0; 0; 1; 1; 0; 0] @ write_black_code (n - 768)
+  | n when n >= 704 -> [0; 0; 0; 0; 0; 0; 1; 0; 0; 1; 0; 1; 1] @ write_black_code (n - 704)
+  | n when n >= 640 -> [0; 0; 0; 0; 0; 0; 1; 0; 0; 1; 0; 1; 0] @ write_black_code (n - 640)
+  | n when n >= 576 -> [0; 0; 0; 0; 0; 0; 1; 1; 0; 1; 1; 0; 1] @ write_black_code (n - 576)
+  | n when n >= 512 -> [0; 0; 0; 0; 0; 0; 1; 1; 0; 1; 1; 0; 0] @ write_black_code (n - 512)
+  | n when n >= 448 -> [0; 0; 0; 0; 0; 0; 1; 1; 0; 1; 0; 1] @ write_black_code (n - 448)
+  | n when n >= 384 -> [0; 0; 0; 0; 0; 0; 1; 1; 0; 1; 0; 0] @ write_black_code (n - 384)
+  | n when n >= 320 -> [0; 0; 0; 0; 0; 0; 1; 1; 0; 0; 1; 1] @ write_black_code (n - 320)
+  | n when n >= 256 -> [0; 0; 0; 0; 0; 1; 0; 1; 1; 0; 1; 1] @ write_black_code (n - 256)
+  | n when n >= 192 -> [0; 0; 0; 0; 1; 1; 0; 0; 1; 0; 0; 1] @ write_black_code (n - 192)
+  | n when n >= 128 -> [0; 0; 0; 0; 1; 1; 0; 0; 1; 0; 0; 0] @ write_black_code (n - 128)
+  | n when n >= 64 -> [0; 0; 0; 0; 0; 0; 1; 1; 1; 1] @  write_black_code (n - 64)
+  | _ -> raise (Pdf.PDFError "write_black_code: unknown")
+
 let rec read_white_code i =
   let a = getbitint i in
   let b = getbitint i in
@@ -855,6 +1071,32 @@ let decode_CCITTFax k eol eba c r eob bone dra input =
             | End_of_file -> bytes_of_write_bitstream output
             | _ -> raise (Failure "Bad CCITT Stream") 
 
+(* CCITT Group 4 encoder *)
+
+(* Data comes in as bytes, with each scanline padded with zeroes. In addition,
+   it is padded to bytes at the end. (Why though? Check for our case.)
+
+Output is suitable for /CCITTFaxDecode /Columns <columns> /K -1 with all other
+dictionary entries as default. i.e:
+
+/EndOfLine false
+/EncodedByteAlign false
+/Rows 0
+/EndOfBlock true
+/BlackIs1 false
+/DamagedRowsBeforeError 0 *)
+
+let encode_ccitt columns stream = stream
+
+(* Tester. *)
+(*let _ =
+  Printf.printf "Running test_encode_ccitt\n";
+  let data = contents_of_file "/Users/john/Desktop/CCITT/out.dat" in
+  let g4 = encode_ccitt 2592 (Pdfio.bytes_of_string data) in
+  let fh = open_out_bin "/Users/john/Desktop/CCITT/g4.dat" in
+    output_string fh (Pdfio.string_of_bytes g4);
+    close_out fh*)
+ 
 (* PNG and TIFF Predictors *)
 
 (* Get the value at index i from an int array a, giving zero if the index is
@@ -981,7 +1223,7 @@ let encode_predictor pred colors bpc columns stream =
             for byte = 0 to columns * 3 - 1 do
               o.Pdfio.output_byte
                 ((get0 stream (scanline * columns + byte)) -
-                 (get0 stream (scanline * columns + byte - 3))
+                 (if byte < 3 then 0 else get0 stream (scanline * columns + byte - 3))
                  mod 256)
             done
           done;
