@@ -28,6 +28,3 @@ val pdfobject_of_destination : t -> Pdf.pdfobject
 
 (** Transform a destination by a matrix *)
 val transform_destination : Pdftransform.transform_matrix -> t -> t
-
-(**/**)
-val string_of_destination : t -> string
