@@ -2,6 +2,7 @@ open Pdfutil
 open Pdfio
 
 let debug = ref false
+let write_comments = ref false
 
 (* Graphics operators. *)
 type t =
@@ -79,13 +80,14 @@ type t =
   | Op_BX (* Start compatibility mode *)
   | Op_EX (* End compatibility mode *)
   | Op_Unknown of string (* Unknown operand / operator sequence *)
+  | Op_Comment of string (* Comments are silently ignored when reading, but can be written for debug. *)
 
 type lexeme =
   | Op of string
   | Obj of Pdfgenlex.t
   | PdfObj of Pdf.pdfobject
   | LexInlineImage of (Pdf.pdfobject * Pdf.pdfobject option * bytes)
-  | LexComment
+  | LexComment of string
   
 (* Lexing *)
 let lexemes_of_op f = function
@@ -217,6 +219,7 @@ let lexemes_of_op f = function
   | Op_BX -> f (Op "BX")
   | Op_EX -> f (Op "EX")
   | Op_Unknown _ -> ()
+  | Op_Comment s -> f (Obj (Pdfgenlex.LexComment s)) 
 
 let lexemes_of_ops ops =
   let ls = ref [] in
@@ -239,7 +242,7 @@ let rec filterspecial = function
 let b = Buffer.create 256
 
 let string_of_lexeme = function
-  | LexComment -> ""
+  | LexComment s -> ""
   | Obj o -> Pdfread.string_of_lexeme o 
   | Op op -> op
   | PdfObj obj -> Pdfwrite.string_of_pdf obj
@@ -251,8 +254,9 @@ let string_of_lexeme = function
             (* Was there previously a predictor? *)
             let dict, data =
               match dp with
-              | None -> dict, data
-              | Some d ->
+              | _ -> dict, data
+              (* 13/10/2024: This is failing, removed until fixed: https://github.com/coherentgraphics/cpdf-binaries/issues/90 *)
+              (*| Some d ->
                   let colours = match Pdf.lookup_direct (Pdf.empty ()) "/Colors" d with Some (Pdf.Integer x) -> x | _ -> 1 in
                   let bpc = match Pdf.lookup_direct_orelse (Pdf.empty ()) "/BPC" "/BitsPerComponent" d with Some (Pdf.Integer x) -> x | _ -> 8 in
                   let columns = match Pdf.lookup_direct (Pdf.empty ()) "/Columns" d with Some (Pdf.Integer x) -> x | _ -> 1 in
@@ -266,7 +270,7 @@ let string_of_lexeme = function
                                              ("/Predictor", Pdf.Integer 11)])
                         in
                           dict, data
-                    | _, _ -> dict, data
+                    | _, _ -> dict, data*)
             in
               Pdf.add_dict_entry dict "/F" (Pdf.Name "/Fl"),
               Pdfcodec.encode_flate data
@@ -601,7 +605,7 @@ let lex_next pdf resources i =
             | '<' -> get_dictionary i
             | _ -> lex_hexstring i
             end
-        | '%' -> ignore (Pdfread.lex_comment i); LexComment
+        | '%' -> ignore (Pdfread.lex_comment i); LexComment ""
         | _ -> raise (Pdf.PDFError "Lexing failure in content stream")
   with
     | Pdf.PDFError r -> 
@@ -624,7 +628,7 @@ let lex_stream pdf resources i =
     try
       while true do
         match lex_next pdf resources i with
-        | LexComment -> ()
+        | LexComment _ -> ()
         | lexeme -> lexemes := lexeme::!lexemes
       done;
       []
@@ -872,24 +876,26 @@ let parse_operators pdf resources streams =
     parse_stream pdf resources rawstreams
     
 (* Flattening *)
-let whitespace = " " (* Change to "\n" if desired. May be user-settable in future. *)
+let whitespace = ref " "
+let always_add_whitespace = ref false
 
 (* Give a bigarray representing a list of graphics operators. *)
 let stream_of_lexemes (oplists : lexeme list list) =
   let strings = map string_of_lexemes oplists in
-    (* Insert a space if the neither the last character of a string nor the
+    (* Insert whitespace if the neither the last character of a string nor the
      * first character of the next is a delimiter *)
     let rec addspaces prev = function
       [] -> rev prev
     | [x] -> addspaces (x::prev) []
     | x::y::r ->
         if
-             String.length x > 0 && Pdf.is_delimiter x.[String.length x - 1]
-          || String.length y > 0 && Pdf.is_delimiter y.[0]
+             (String.length x > 0 && Pdf.is_delimiter x.[String.length x - 1]
+          || String.length y > 0 && Pdf.is_delimiter y.[0])
+          && not !always_add_whitespace
         then
           addspaces (x::prev) (y::r)
         else
-          addspaces (whitespace::x::prev) (y::r)
+          addspaces (!whitespace::x::prev) (y::r)
     in
       let strings = addspaces [] strings in
         let total_length =
