@@ -118,7 +118,10 @@ val write_font : ?objnum:int -> Pdf.t -> font -> int
 
 (** {2 Utility functions} *)
 
+(** Is a PDF string UTF16be (i.e does it have a byte order marker at the beginning)? *)
 val is_unicode : string -> bool
+
+(** Is a font Identity H? *)
 val is_identity_h : font -> bool
 
 (** A list of unicode codepoints for a UTF8 string *)
@@ -190,3 +193,6 @@ val table_of_encoding : encoding -> (int, string) Hashtbl.t
 
 (** Reverse table of all the entries in an encoding. *)
 val reverse_table_of_encoding : encoding -> (string, int) Hashtbl.t
+
+(** Parse a [/ToUnicode] entry. *)
+val parse_tounicode : Pdf.t -> Pdf.pdfobject -> (int * string) list
