@@ -197,14 +197,6 @@ and strings_of_pdf ?(hex=false) f changetable = function
         f (WString (string_of_int n'));
         f (WString " 0 R")
 
-let strings_of_pdf_return obj =
-  let strings = ref [] in
-    strings_of_pdf
-      (function x -> strings := x::!strings)
-      (Hashtbl.create 0)
-      obj;
-    rev !strings
-
 (* Produce a single string from a PDF object. *)
 let b = Buffer.create 100
 
@@ -239,17 +231,6 @@ let debug_whole_pdf pdf =
   Pdfe.log (Printf.sprintf "trailerdict = %s\n" (string_of_pdf pdf.Pdf.trailerdict));
   Pdf.objiter (fun i o -> Pdfe.log (Printf.sprintf "%i = %s\n" i (string_of_pdf o))) pdf
 
-(* Calculate strings, one for each indirect object in the body. *)
-let strings_of_object (n, pdfobject) =
-  let strings = ref [] in
-  strings := [WString (string_of_int n); WString " 0 obj\n"];
-  strings_of_pdf
-    (function x -> strings := x::!strings)
-    (Hashtbl.create 0)
-    pdfobject;
-  strings := WString "\nendobj\n"::!strings;
-  rev !strings
-
 let strings_of_pdf_object f (_, pdfobject) n' changetable =
   f (WString (string_of_int n'));
   f (WString " 0 obj\n");
@@ -333,7 +314,7 @@ let bake_object_streams compress pdf numbers =
               ("/First", Pdf.Integer first)]
          in
            let obj = Pdf.Stream {contents = (dict, Pdf.Got data)} in
-             if compress then Pdfcodec.encode_pdfstream pdf Pdfcodec.Flate obj;
+             if compress then Pdfcodec.encode_pdfstream ~only_if_smaller:true pdf Pdfcodec.Flate obj;
              Pdf.addobj_given_num pdf (tostream, obj))
     numbers
 
@@ -476,6 +457,7 @@ let make_xref_stream pdf xrefs renumbered_objects_for_streams =
           in
             let xrefstream = Pdf.Stream {contents = (dict, Pdf.Got data)} in
               Pdfcodec.encode_pdfstream
+                ~only_if_smaller:true
                 pdf
                 Pdfcodec.Flate
                 ~predictor:Pdfcodec.PNGUp
